@@ -4,7 +4,7 @@
 
 use std::fs;
 
-use quickdna::trans_table::{CodonIdx, Nucleotide, TranslationTable};
+use quickdna::trans_table::{Codon, CodonIdx, Nucleotide, TranslationTable};
 
 // via https://en.wikipedia.org/wiki/List_of_genetic_codes
 const TABLE: &str = "
@@ -74,6 +74,49 @@ GGA || G || G || G || G || G || G || G || G || G || G || G || G || G || G || G |
 GGG || G || G || G || G || G || G || G || G || G || G || G || G || G || G || G || G || G || G || G || G || G || G || G || G || G || G || G
 ";
 
+fn codon_first_n_index(codon: Codon) -> Option<usize> {
+    let codon: [Nucleotide; 3] = codon.into();
+    codon
+        .iter()
+        .enumerate()
+        .find_map(|(idx, &n)| if n == Nucleotide::N { Some(idx) } else { None })
+}
+
+fn ambiguous_codon_protein(
+    codon: Codon,
+    table_idx: usize,
+    translation_tables: [u8; TranslationTable::LOOKUP_SIZE],
+) -> u8 {
+    match codon.count(Nucleotide::N) {
+        0 => panic!("not an ambigious codon: {}", codon),
+        1 => {
+            // try and see if all possible permutations of this codon map to the same protein
+            let mut seen_protein = None;
+            let n_index = codon_first_n_index(codon).unwrap();
+
+            for possible in [Nucleotide::A, Nucleotide::T, Nucleotide::C, Nucleotide::G] {
+                let mut candidate: [Nucleotide; 3] = codon.into();
+                candidate[n_index] = possible;
+                let codon_idx: usize = CodonIdx::from(candidate).into();
+                let candidate_protein =
+                    translation_tables[table_idx * TranslationTable::CODONS_PER_TABLE + codon_idx];
+
+                match seen_protein {
+                    None => {
+                        seen_protein = Some(candidate_protein);
+                    }
+                    Some(p) if p == candidate_protein => {}
+                    _ => return b'X',
+                }
+            }
+
+            seen_protein.unwrap()
+        }
+        // all codons with 2 or more Ns always map to X
+        _ => b'X',
+    }
+}
+
 fn gen_translation_tables() -> [u8; TranslationTable::LOOKUP_SIZE] {
     let mut translation_tables = [b'*'; TranslationTable::LOOKUP_SIZE];
 
@@ -94,6 +137,24 @@ fn gen_translation_tables() -> [u8; TranslationTable::LOOKUP_SIZE] {
         }
     }
 
+    // setup ambiguous codons
+    for &a in Nucleotide::NUCLEOTIDES.iter() {
+        for &b in Nucleotide::NUCLEOTIDES.iter() {
+            for &c in Nucleotide::NUCLEOTIDES.iter() {
+                if a == Nucleotide::N || b == Nucleotide::N || c == Nucleotide::N {
+                    let codon = Codon([a, b, c]);
+                    let codon_idx: usize = CodonIdx::from(codon).into();
+
+                    for table_idx in 0..TranslationTable::N_TRANS_TABLES {
+                        let protein = ambiguous_codon_protein(codon, table_idx, translation_tables);
+                        translation_tables
+                            [table_idx * TranslationTable::CODONS_PER_TABLE + codon_idx] = protein;
+                    }
+                }
+            }
+        }
+    }
+
     translation_tables
 }
 
