@@ -24,9 +24,14 @@ pub enum Nucleotide {
     T = 1,
     C = 2,
     G = 3,
+    /// IUPAC ambiguity code
+    N = 4,
 }
 
 impl Nucleotide {
+    pub const NUCLEOTIDES: [Self; 5] = [Self::A, Self::T, Self::C, Self::G, Self::N];
+    const N_NUCLEOTIDES: u8 = Self::NUCLEOTIDES.len() as u8;
+
     const fn ascii_pack_table() -> [u8; 128] {
         let mut pack_table = [255u8; 128];
         pack_table[b'a' as usize] = Self::A as u8;
@@ -37,18 +42,20 @@ impl Nucleotide {
         pack_table[b'C' as usize] = Self::C as u8;
         pack_table[b'g' as usize] = Self::G as u8;
         pack_table[b'G' as usize] = Self::G as u8;
+        pack_table[b'n' as usize] = Self::N as u8;
+        pack_table[b'N' as usize] = Self::N as u8;
         pack_table
     }
 
     const ASCII_PACK_TABLE: [u8; 128] = Self::ascii_pack_table();
 
-    const COMPLEMENT_TABLE: [Self; 4] = [Self::T, Self::A, Self::G, Self::C];
-
+    const COMPLEMENT_TABLE: [Self; Self::N_NUCLEOTIDES as usize] =
+        [Self::T, Self::A, Self::G, Self::C, Self::N];
     pub fn complement(self) -> Self {
         Self::COMPLEMENT_TABLE[self as u8 as usize]
     }
 
-    const ASCII_MAP: [u8; 4] = [b'A', b'T', b'C', b'G'];
+    const ASCII_MAP: [u8; Self::N_NUCLEOTIDES as usize] = [b'A', b'T', b'C', b'G', b'N'];
     pub fn to_ascii(self) -> u8 {
         Self::ASCII_MAP[self as usize]
     }
@@ -61,8 +68,9 @@ impl TryFrom<u8> for Nucleotide {
     fn try_from(u: u8) -> Result<Self, Self::Error> {
         if u < 128 {
             let v = Self::ASCII_PACK_TABLE[u as usize];
-            if v < 4 {
-                // SAFETY: there are only 4 variants, with assigned numbers, so 0..=3 are valid reprs of this type
+            if v < Self::N_NUCLEOTIDES {
+                // SAFETY: there are only X variants, with assigned numbers, so 0..=(X - 1)
+                //         are valid reprs of this type
                 Ok(unsafe { std::mem::transmute(v) })
             } else {
                 Err(cold_error(TranslationError::BadNucleotide(u.into())))
@@ -80,16 +88,81 @@ impl fmt::Display for Nucleotide {
             Self::T => write!(f, "T"),
             Self::C => write!(f, "C"),
             Self::G => write!(f, "G"),
+            Self::N => write!(f, "N"),
         }
     }
 }
 
+#[derive(Debug, Clone, Copy, PartialEq, Eq, std::hash::Hash)]
+pub struct Codon(pub [Nucleotide; 3]);
+
+impl Codon {
+    /// How many of the specified nucleotide this codon includes
+    pub fn count(&self, n: Nucleotide) -> usize {
+        let mut count = 0;
+        if self.0[0] == n {
+            count += 1
+        }
+        if self.0[1] == n {
+            count += 1
+        }
+        if self.0[2] == n {
+            count += 1
+        }
+        count
+    }
+
+    /// Whether the codon includes `N`
+    pub fn is_ambiguous(&self) -> bool {
+        self.count(Nucleotide::N) > 0
+    }
+
+    /// Returns an iterator of all codons, including ambiguous codons with `N`
+    pub fn all_codons() -> impl Iterator<Item = Self> {
+        const N: [Nucleotide; 5] = Nucleotide::NUCLEOTIDES;
+        N.iter().flat_map(move |&a| {
+            N.iter()
+                .flat_map(move |&b| N.iter().map(move |&c| Codon([a, b, c])))
+        })
+    }
+}
+
+impl TryFrom<[u8; 3]> for Codon {
+    type Error = TranslationError;
+
+    fn try_from(value: [u8; 3]) -> Result<Self, Self::Error> {
+        Ok(Self([
+            Nucleotide::try_from(value[0])?,
+            Nucleotide::try_from(value[1])?,
+            Nucleotide::try_from(value[2])?,
+        ]))
+    }
+}
+
+impl From<Codon> for [Nucleotide; 3] {
+    fn from(c: Codon) -> Self {
+        c.0
+    }
+}
+
+impl fmt::Display for Codon {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        write!(f, "{}{}{}", self.0[0], self.0[1], self.0[2])
+    }
+}
+
 #[repr(transparent)]
 pub struct CodonIdx(usize);
 
 impl From<[Nucleotide; 3]> for CodonIdx {
     fn from(value: [Nucleotide; 3]) -> Self {
-        Self((value[0] as usize) << 4 | (value[1] as usize) << 2 | (value[2] as usize))
+        Self((value[0] as usize) << 6 | (value[1] as usize) << 3 | (value[2] as usize))
+    }
+}
+
+impl From<Codon> for CodonIdx {
+    fn from(c: Codon) -> Self {
+        c.0.into()
     }
 }
 
@@ -134,10 +207,12 @@ pub enum TranslationTable {
 }
 
 impl TranslationTable {
-    /// Number of codons per table
-    pub const CODONS_PER_TABLE: usize = 64;
-    /// N of codons * N of tables
-    pub const LOOKUP_SIZE: usize = Self::CODONS_PER_TABLE * 27;
+    /// There are really only 125 possible codons (len(ATCGN)^3), but since codons take up
+    /// 3 bits, the maximum codon value is 0b100100100, so we need some holes in the table.
+    pub const CODONS_PER_TABLE: usize = 293;
+    // Number of NCBI translation tables (they go up to 33, but there's gaps in the numbering)
+    pub const N_TRANS_TABLES: usize = 27;
+    pub const LOOKUP_SIZE: usize = Self::CODONS_PER_TABLE * Self::N_TRANS_TABLES;
     /// Generated by bin/gen_table.rs, layout is all codons for table 1, then all codons for table 2, etc.
     const TRANSLATION_TABLES: &'static [u8; Self::LOOKUP_SIZE] = include_bytes!("tables.dat");
 
@@ -192,7 +267,10 @@ impl TranslationTable {
             let b = chunk[1].try_into()?;
             let c = chunk[2].try_into()?;
             let codon_idx = CodonIdx::from([a, b, c]);
-            result.push(Self::TRANSLATION_TABLES[table_idx * 64 + codon_idx.0]);
+            result.push(
+                Self::TRANSLATION_TABLES
+                    [table_idx * TranslationTable::CODONS_PER_TABLE + codon_idx.0],
+            );
         }
 
         Ok(result)
